
// Aceasta este biblioteca pentru I2C
#include "Wire.h"

// Aceasta este biblioteca pentru LCD
#include "LCD.h" 

#include "LiquidCrystal_I2C.h"

// Definesc pinul pentru buton.
#define PIN_BUTON 2
// Definesc pinul pentru autoplay.
#define PIN_AUTOPLAY 1
// Definesc valoarea pinului pentru buzzer.
#define PIN_BUZZER 5
// Definesc o variabila de tip int care numara secundele 
// care trec de la inceperea jocului.
static int contor = 0;

// Aici definesc o variabila de tip contor pentru
// melodie.
int thisNote = 0;

#define ALERGARE_1 1
#define ALERGARE_2 2
#define SARITURA 3
#define SARITURA_SUS '.' // Caracterul '.' pentru capul jucatorului
#define SARITURA_JOS 4
#define CELULA_GOALA ' '   // Caracterul spatiu (gol)
#define BLOC_COMPLET 5
#define BLOC_DREAPTA 6
#define BLOC_STANGA 7

#define POZITIE_ORIZONTALA_JUCATOR 1       // Pozitie orizontala pentru jucator pe ecran
#define BLOC_SUS 1
#define BLOC_JOS 2
#define POZITIE_JUCATOR_REPAUS 0           // Jucatorul se afla in repaus
#define POZITIE_JUCATOR_JOS_1 1            // Jucatorul se afla pe randul de jos, pozitia 1
#define POZITIE_JUCATOR_JOS_2 2            // Jucatorul se afla pe randul de jos, pozitia 2

#define POZITIE_JUCATOR_ALERGARE_1 3       // Prima pozitie pentru alergare
#define POZITIE_JUCATOR_ALERGARE_2 4       // A doua pozitie pentru alergare

#define POZITIE_JUCATOR_SARITURA_1 5       // Prima pozitie pentru saritura (pe randul de jos)
#define POZITIE_JUCATOR_SARITURA_2 6       // A doua pozitie pentru saritura (pe ambele randuri)
#define POZITIE_JUCATOR_SARITURA_3 7       // A treia pozitie pentru saritura (pe randul de sus)

#define POZITIE_JUCATOR_CAZATURA_1 8       // Prima pozitie pentru cazatura (pe randul de sus)
#define POZITIE_JUCATOR_CAZATURA_2 9       // A doua pozitie pentru cazatura (pe cele doua randuri, la coborare)
#define POZITIE_JUCATOR_CAZATURA_3 10      // A treia pozitie pentru saritura (pe randul de jos)

#define POZITIE_JUCATOR_SUS_1 11           // Jucatorul se afla pe randul de sus, pozitia 1
#define POZITIE_JUCATOR_SUS_2 12           // Jucatorul se afla pe randul de sus, pozitia 2

LiquidCrystal_I2C lcd(0x27,2,1,0,4,5,6,7);

// Aici reprezint obstacolele ca doi vectori pe cele doua linii de pe LCD (cea de sus si cea de jos).
static char harta_sus[17];
static char harta_jos[17];

// Aici retin starea butonului.
static int buton_activat = 0;
// Aici retin daca jocul a inceput sau nu.
static bool rulare_joc = false;
// Aici retin daca ecranul clipeste sau nu.
static bool mesaj_inceput_joc = true;

// Functie de initializare a hartii.
void initializare_harta() {
  int index;
  // Definesc o matrice pentru cadrele pe care le voi desena pe ecran.
  static byte grafica[] = {
    // Pozitie 1 alergare
    B00000,
    B01100,
    B01100,
    B01110,
    B11100,
    B01100,
    B11010,
    B10011,
    // Pozitie 2 alergare
    B00000,
    B01100,
    B01100,
    B01100,
    B01100,
    B01100,
    B01100,
    B01110,
    // Saritura
    B00000,
    B01100,
    B01100,
    B11110,
    B01101,
    B11111,
    B10000,
    B00000,
    // Saritura
    B11110,
    B01101,
    B11111,
    B10000,
    B00000,
    B00000,
    B00000,
    B00000,
    // Obstacol complet
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    B11111,
    // Obstacol dreapta
    B00011,
    B00011,
    B00011,
    B00011,
    B00011,
    B00011,
    B00011,
    B00011,
    // Obstacol stanga
    B11000,
    B11000,
    B11000,
    B11000,
    B11000,
    B11000,
    B11000,
    B11000,
  };
  
  index = 0;
  // Creez fiecare din cele 7 cadre reprezentate in matricea grafica.
  while (index < 7) {
    lcd.createChar(index + 1, &grafica[index * 8]);
    index ++;  
  }

  // Initializez ecranul cu celule goale.
  index = 0;
  while (index < 16) {
    harta_sus[index] = CELULA_GOALA;
    harta_jos[index] = CELULA_GOALA;
    index++;  
  }
}

// Aici am definite niste macro-uri pentru notele muzicale 
// asociate melodiei.
#define NOTE_B0  31
#define NOTE_C1  33
#define NOTE_CS1 35
#define NOTE_D1  37
#define NOTE_DS1 39
#define NOTE_E1  41
#define NOTE_F1  44
#define NOTE_FS1 46
#define NOTE_G1  49
#define NOTE_GS1 52
#define NOTE_A1  55
#define NOTE_AS1 58
#define NOTE_B1  62
#define NOTE_C2  65
#define NOTE_CS2 69
#define NOTE_D2  73
#define NOTE_DS2 78
#define NOTE_E2  82
#define NOTE_F2  87
#define NOTE_FS2 93
#define NOTE_G2  98
#define NOTE_GS2 104
#define NOTE_A2  110
#define NOTE_AS2 117
#define NOTE_B2  123
#define NOTE_C3  131
#define NOTE_CS3 139
#define NOTE_D3  147
#define NOTE_DS3 156
#define NOTE_E3  165
#define NOTE_F3  175
#define NOTE_FS3 185
#define NOTE_G3  196
#define NOTE_GS3 208
#define NOTE_A3  220
#define NOTE_AS3 233
#define NOTE_B3  247
#define NOTE_C4  262
#define NOTE_CS4 277
#define NOTE_D4  294
#define NOTE_DS4 311
#define NOTE_E4  330
#define NOTE_F4  349
#define NOTE_FS4 370
#define NOTE_G4  392
#define NOTE_GS4 415
#define NOTE_A4  440
#define NOTE_AS4 466
#define NOTE_B4  494
#define NOTE_C5  523
#define NOTE_CS5 554
#define NOTE_D5  587
#define NOTE_DS5 622
#define NOTE_E5  659
#define NOTE_F5  698
#define NOTE_FS5 740
#define NOTE_G5  784
#define NOTE_GS5 831
#define NOTE_A5  880
#define NOTE_AS5 932
#define NOTE_B5  988
#define NOTE_C6  1047
#define NOTE_CS6 1109
#define NOTE_D6  1175
#define NOTE_DS6 1245
#define NOTE_E6  1319
#define NOTE_F6  1397
#define NOTE_FS6 1480
#define NOTE_G6  1568
#define NOTE_GS6 1661
#define NOTE_A6  1760
#define NOTE_AS6 1865
#define NOTE_B6  1976
#define NOTE_C7  2093
#define NOTE_CS7 2217
#define NOTE_D7  2349
#define NOTE_DS7 2489
#define NOTE_E7  2637
#define NOTE_F7  2794
#define NOTE_FS7 2960
#define NOTE_G7  3136
#define NOTE_GS7 3322
#define NOTE_A7  3520
#define NOTE_AS7 3729
#define NOTE_B7  3951
#define NOTE_C8  4186
#define NOTE_CS8 4435
#define NOTE_D8  4699
#define NOTE_DS8 4978
#define REST      0

// Aici este definit tempo-ul melodiei.
int tempo = 200;
// Aici retin valoarea pinului pentru buzzer.
int buzzer = 5;

// Aici este codul asociat melodiei.
int melody[] = {
  // Super Mario Bros theme
  // Score available at https://musescore.com/user/2123/scores/2145
  // Theme by Koji Kondo
  
  NOTE_E5,8, NOTE_E5,8, REST,8, NOTE_E5,8, REST,8, NOTE_C5,8, NOTE_E5,8, //1
  NOTE_G5,4, REST,4, NOTE_G4,8, REST,4, 
  NOTE_C5,-4, NOTE_G4,8, REST,4, NOTE_E4,-4, // 3
  NOTE_A4,4, NOTE_B4,4, NOTE_AS4,8, NOTE_A4,4,
  NOTE_G4,-8, NOTE_E5,-8, NOTE_G5,-8, NOTE_A5,4, NOTE_F5,8, NOTE_G5,8,
  REST,8, NOTE_E5,4,NOTE_C5,8, NOTE_D5,8, NOTE_B4,-4,
  NOTE_C5,-4, NOTE_G4,8, REST,4, NOTE_E4,-4, // repeats from 3
  NOTE_A4,4, NOTE_B4,4, NOTE_AS4,8, NOTE_A4,4,
  NOTE_G4,-8, NOTE_E5,-8, NOTE_G5,-8, NOTE_A5,4, NOTE_F5,8, NOTE_G5,8,
  REST,8, NOTE_E5,4,NOTE_C5,8, NOTE_D5,8, NOTE_B4,-4,

  REST,4, NOTE_G5,8, NOTE_FS5,8, NOTE_F5,8, NOTE_DS5,4, NOTE_E5,8,//7
  REST,8, NOTE_GS4,8, NOTE_A4,8, NOTE_C4,8, REST,8, NOTE_A4,8, NOTE_C5,8, NOTE_D5,8,
  REST,4, NOTE_DS5,4, REST,8, NOTE_D5,-4,
  NOTE_C5,2, REST,2,

  REST,4, NOTE_G5,8, NOTE_FS5,8, NOTE_F5,8, NOTE_DS5,4, NOTE_E5,8,//repeats from 7
  REST,8, NOTE_GS4,8, NOTE_A4,8, NOTE_C4,8, REST,8, NOTE_A4,8, NOTE_C5,8, NOTE_D5,8,
  REST,4, NOTE_DS5,4, REST,8, NOTE_D5,-4,
  NOTE_C5,2, REST,2,

  NOTE_C5,8, NOTE_C5,4, NOTE_C5,8, REST,8, NOTE_C5,8, NOTE_D5,4,//11
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2,

  NOTE_C5,8, NOTE_C5,4, NOTE_C5,8, REST,8, NOTE_C5,8, NOTE_D5,8, NOTE_E5,8,//13
  REST,1, 
  NOTE_C5,8, NOTE_C5,4, NOTE_C5,8, REST,8, NOTE_C5,8, NOTE_D5,4,
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2,
  NOTE_E5,8, NOTE_E5,8, REST,8, NOTE_E5,8, REST,8, NOTE_C5,8, NOTE_E5,4,
  NOTE_G5,4, REST,4, NOTE_G4,4, REST,4, 
  NOTE_C5,-4, NOTE_G4,8, REST,4, NOTE_E4,-4, // 19
  
  NOTE_A4,4, NOTE_B4,4, NOTE_AS4,8, NOTE_A4,4,
  NOTE_G4,-8, NOTE_E5,-8, NOTE_G5,-8, NOTE_A5,4, NOTE_F5,8, NOTE_G5,8,
  REST,8, NOTE_E5,4, NOTE_C5,8, NOTE_D5,8, NOTE_B4,-4,

  NOTE_C5,-4, NOTE_G4,8, REST,4, NOTE_E4,-4, // repeats from 19
  NOTE_A4,4, NOTE_B4,4, NOTE_AS4,8, NOTE_A4,4,
  NOTE_G4,-8, NOTE_E5,-8, NOTE_G5,-8, NOTE_A5,4, NOTE_F5,8, NOTE_G5,8,
  REST,8, NOTE_E5,4, NOTE_C5,8, NOTE_D5,8, NOTE_B4,-4,

  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,//23
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_D5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_G5,-8, NOTE_F5,-8,
  
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2, //26
  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_B4,8, NOTE_F5,4, NOTE_F5,8, NOTE_F5,-8, NOTE_E5,-8, NOTE_D5,-8,
  NOTE_C5,8, NOTE_E4,4, NOTE_E4,8, NOTE_C4,2,

  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,//repeats from 23
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_D5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_G5,-8, NOTE_F5,-8,
  
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2, //26
  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_B4,8, NOTE_F5,4, NOTE_F5,8, NOTE_F5,-8, NOTE_E5,-8, NOTE_D5,-8,
  NOTE_C5,8, NOTE_E4,4, NOTE_E4,8, NOTE_C4,2,
  NOTE_C5,8, NOTE_C5,4, NOTE_C5,8, REST,8, NOTE_C5,8, NOTE_D5,8, NOTE_E5,8,
  REST,1,

  NOTE_C5,8, NOTE_C5,4, NOTE_C5,8, REST,8, NOTE_C5,8, NOTE_D5,4, //33
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2,
  NOTE_E5,8, NOTE_E5,8, REST,8, NOTE_E5,8, REST,8, NOTE_C5,8, NOTE_E5,4,
  NOTE_G5,4, REST,4, NOTE_G4,4, REST,4, 
  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_D5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_A5,-8, NOTE_G5,-8, NOTE_F5,-8,
  
  NOTE_E5,8, NOTE_C5,4, NOTE_A4,8, NOTE_G4,2, //40
  NOTE_E5,8, NOTE_C5,4, NOTE_G4,8, REST,4, NOTE_GS4,4,
  NOTE_A4,8, NOTE_F5,4, NOTE_F5,8, NOTE_A4,2,
  NOTE_B4,8, NOTE_F5,4, NOTE_F5,8, NOTE_F5,-8, NOTE_E5,-8, NOTE_D5,-8,
  NOTE_C5,8, NOTE_E4,4, NOTE_E4,8, NOTE_C4,2,
  
  //game over sound
  NOTE_C5,-4, NOTE_G4,-4, NOTE_E4,4, //45
  NOTE_A4,-8, NOTE_B4,-8, NOTE_A4,-8, NOTE_GS4,-8, NOTE_AS4,-8, NOTE_GS4,-8,
  NOTE_G4,8, NOTE_D4,8, NOTE_E4,-2,  
};

// sizeof gives the number of bytes, each int value is composed of two bytes (16 bits)
// there are two values per note (pitch and duration), so for each note there are four bytes
int notes = sizeof(melody) / sizeof(melody[0]) / 2;

// this calculates the duration of a whole note in ms
int wholenote = (60000 * 4) / tempo;
int divider = 0, noteDuration = 0;

// In aceasta functie derulez pasul urmator al hartii.
void animatieHarta(char* harta, byte cadru_nou) {
  int index = 0;
  char patrat_urmator, patrat_curent;

  // Pentru fiecare patrat din cele 16 de pe o linie
  while (index < 16) {
    patrat_curent = harta[index];
    
    if (index == 15) {
      patrat_urmator = cadru_nou; 
    } else {
      patrat_urmator = harta[index + 1];
    }
    
    switch (patrat_curent) {
      // Daca patratul curent este o celula goala
      case CELULA_GOALA:
        if (patrat_urmator == BLOC_COMPLET) {
          // Daca patratul urmator este un patrat plin, atunci, la urmatorul cadru,
          // se va muta cu o pozitie la stanga, deci patratul curent va deveni 
          // plin doar pe jumatatea dreapta.
          harta[index] = BLOC_DREAPTA;
        } else {
          // Daca patratul urmator este plin doar pe jumatate (cea dreapta), atunci, la urmatorul cadru,
          // prin deplasarea patratelor catre stanga, patratul curent va ramane gol, iar daca patratul 
          // urmator era gol deja, patratul curent va ramane gol. 
          harta[index] = CELULA_GOALA;  
        }
        
        break;
      // Daca patratul curent este plin
      case BLOC_COMPLET:
        // Daca patratul urmator este gol, prin deplasarea patratelor catre stanga,
        // va fi plin doar pe jumatate (jumatatea stanga).
        if (patrat_urmator == CELULA_GOALA) {
          harta[index] = BLOC_STANGA;  
        } else {
          // Daca urmatorul patrat nu este gol, atunci, prin deplasarea patratelor catre
          // stanga, acesta va ramane plin la urmatorul cadru.
          harta[index] = BLOC_COMPLET;  
        }
        
        break;
      // Daca patratul curent este plin doar pe jumatatea dreapta, inseamna ca
      // dupa el urmeaza un patrat complet, deci, la urmatorul cadru, va ramane plin.
      case BLOC_DREAPTA:
        harta[index] = BLOC_COMPLET;
        break;

      // Daca patratul curent este plin doar pe jumatatea stanga, atunci, inseamna ca 
      // urmatorul patrat va fi gol, deci, la cadrul urmator, va deveni gol.
      case BLOC_STANGA:
        harta[index] = CELULA_GOALA;
        break;
    }

    index++;
  }
}

// In aceasta functie desenez jucatorul pe ecran. Returnez valoarea 0 daca nu am coliziune
// cu obstacolele si 1 in caz contrar.
int reprezentare_jucator(byte pozitie_jucator, char* harta_sus, char* harta_jos, unsigned int timp) {
  int numar_cifre = 0, coliziune = 0;
  // Retin imaginea curenta a LCD-ului (partea de sus si partea de jos).
  char patrat_sus_curent = harta_sus[POZITIE_ORIZONTALA_JUCATOR];
  char patrat_jos_curent = harta_jos[POZITIE_ORIZONTALA_JUCATOR];
  byte patrat_sus_urmator, patrat_jos_urmator;
  
  switch (pozitie_jucator) {
    // Daca jocul nu a inceput si jucatorul de afla in repaus, atunci
    // la pasul urmator nu desenez obstacolele.
    case POZITIE_JUCATOR_REPAUS:
      patrat_sus_urmator = patrat_jos_urmator = CELULA_GOALA;
      break;
      
    // Aici reprezint jucatorul care, in cadrul urmator, intra 
    // in modul de alergare si nu intalneste obstacole sus.
    case POZITIE_JUCATOR_JOS_1:
      patrat_sus_urmator = CELULA_GOALA;
      patrat_jos_urmator = ALERGARE_1;
      break;
      
    // Aici reprezint jucatorul care, in cadrul curent, alearga, iar
    // la cadrul urmator nu intalneste obstacole nici sus nici jos.
    case POZITIE_JUCATOR_JOS_2:
      patrat_sus_urmator = CELULA_GOALA;
      patrat_jos_urmator = ALERGARE_2;
      break;

    // Aici reprezint jucatorul care, la pasul curent alearga,
    // iar la pasul urmator, se pregateste sa sara; 
    // jos va fi reprezentat inceputul sariturii, iar sus va fi o celula goala.
    case POZITIE_JUCATOR_ALERGARE_1:
      patrat_sus_urmator = CELULA_GOALA;
      patrat_jos_urmator = SARITURA;
      break;

    // In cadrul curent, jucatorul se pregateste sa 
    // aterizeze, iar la pasul urmator, cadrul de jos va fi reprezentat
    // de ultima parte a caderii, iar cadrul de sus va fi o celula.
    case POZITIE_JUCATOR_CAZATURA_3:
      patrat_sus_urmator = CELULA_GOALA;
      patrat_jos_urmator = SARITURA;
      break;

    // In cadrul curent, jucatorul se ridica cu picioarele
    // de la pamant si realizeaza o parte din saritura.
    // In cadrul urmator, cele doua celule (de sus si de jos)
    // vor fi reprezentate de corpul jucatorului.
    case POZITIE_JUCATOR_ALERGARE_2:
      patrat_sus_urmator = SARITURA_SUS;
      patrat_jos_urmator = SARITURA_JOS;

    // In cadrul curent, jucatorul se apropie cu picioarele
    // de pamant si realizeaza o parte din cadere.
    // In cadrul urmator, cele doua celule (de sus si de jos)
    // vor fi reprezentate de corpul jucatorului.
    case POZITIE_JUCATOR_CAZATURA_2:
      patrat_sus_urmator = SARITURA_SUS;
      patrat_jos_urmator = SARITURA_JOS;
      break;

    // In cadrul curent, jucatorul a ajuns pe nivelul de sus
    // in pozitia de saritura (cu picioarele desfacute).
    // In cadrul urmator, in partea de sus o sa fie aceeasi pozitie,
    // iar jos nu va fi niciun obstacol.
    case POZITIE_JUCATOR_SARITURA_1:
      patrat_sus_urmator = SARITURA;
      patrat_jos_urmator = CELULA_GOALA;
      break;

    // In cadrul curent, jucatorul se afla pe ambele randuri, la mijlocul sariturii,
    // iar la urmatorul cadru, va continua animatia de saritura, situandu-se complet
    // in celula de sus.
    case POZITIE_JUCATOR_SARITURA_2:
      patrat_sus_urmator = SARITURA;
      patrat_jos_urmator = CELULA_GOALA;
      break;

    // In cadrul curent, jucatorul se afla la finalul animatiei de saritura si 
    // finalizeaza animatia la cadrul urmator, situandu-se in randul de sus al LCD-ului.
    case POZITIE_JUCATOR_SARITURA_3:
      patrat_sus_urmator = SARITURA;
      patrat_jos_urmator = CELULA_GOALA;
      break;

    // In cadrul curent, jucatorul se afla pe linia de sus de pe LCD si
    // simuleaza inceputul cazaturii. In urmatorul cadru, continua animatia de cazatura.
    case POZITIE_JUCATOR_CAZATURA_1:
      patrat_sus_urmator = SARITURA;
      patrat_jos_urmator = CELULA_GOALA;
      break;

    // In cadrul curent, jucatorul este pe linia de sus a LCD-ului.
    // Sus o sa intre in modul de alergare, iar jos va fi o celula goala.
    case POZITIE_JUCATOR_SUS_1:
      patrat_sus_urmator = ALERGARE_1;
      patrat_jos_urmator = CELULA_GOALA;
      break;

    // In cadrul curent, jucatorul este pe linia de sus a LCD-ului.
    // Sus o sa intre in modul de alergare, iar jos va fi o celula goala.
    case POZITIE_JUCATOR_SUS_2:
      patrat_sus_urmator = ALERGARE_2;
      patrat_jos_urmator = CELULA_GOALA;
      break;
  }

  // Daca sus urmeaza un obstacol
  if (patrat_sus_urmator != CELULA_GOALA) {
    harta_sus[POZITIE_ORIZONTALA_JUCATOR] = patrat_sus_urmator;
    if (patrat_sus_curent == CELULA_GOALA) {
      coliziune = 0;  
    } else {
      coliziune = 1;  
    }
  }

  // Daca jos urmeaza un obstacol
  if (patrat_jos_urmator != CELULA_GOALA) {
    harta_jos[POZITIE_ORIZONTALA_JUCATOR] = patrat_jos_urmator;
    if (patrat_jos_curent == CELULA_GOALA) {
      coliziune = 0;  
    } else {
      coliziune = 1;  
    }
  }

  // Calculez pe cate patrate trebuie sa afisez timpul scurs de la 
  // inceperea jocului.
  if (timp > 99) {
    numar_cifre = 3;  
  } else if (timp > 9) {
    numar_cifre = 2;  
  } else if (timp <= 9) {
    numar_cifre = 1;  
  }
  
  // Desenez harta
  harta_jos[16] = '\0';
  harta_sus[16] = '\0';
  char caracter = harta_sus[16 - numar_cifre];
  harta_sus[16 - numar_cifre] = '\0';
  lcd.setCursor(0,0);
  // Desenez partea de sus a hartii.
  lcd.print(harta_sus);
  harta_sus[16 - numar_cifre] = caracter;
  lcd.setCursor(0,1);
  // Desenez partea de jos a hartii, incepand cu pozitia
  // jucatorului, care se afla intotdeauna pe a doua celula de pe
  // randul de jos.
  lcd.print(harta_jos);

  // Pozitionez cursorul astfel incat sa reprezint timer-ul.
  lcd.setCursor(16 - numar_cifre,0);
  lcd.print(timp);
  
 
  harta_sus[POZITIE_ORIZONTALA_JUCATOR] = patrat_sus_curent;
  harta_jos[POZITIE_ORIZONTALA_JUCATOR] = patrat_jos_curent;
  
  return coliziune;
}

void setup(){
  Serial.begin(9600);
  pinMode(PIN_BUZZER, OUTPUT);
  pinMode(PIN_BUTON, INPUT);
  digitalWrite(PIN_BUTON, HIGH);
  pinMode(PIN_AUTOPLAY, OUTPUT);
  digitalWrite(PIN_AUTOPLAY, HIGH);
  
  initializare_harta();
  
  lcd.begin(16, 2);
  lcd.setBacklightPin(3,POSITIVE);
  lcd.setBacklight(HIGH);
  
  cli();

  TCCR1A = 0;
  TCCR1B = 0;
  TCNT1  = 0;
  
  // Generez intreruperea cu o frecventa de 5 Hz pentru timer.
  // Am facut calculele conform formulelor din laborator.
  OCR1A = 12499;            // compare match register 16MHz/256/5Hz-1
  TCCR1B |= (1 << WGM12);   // CTC mode
  TCCR1B |= (1 << CS12);    // 256 prescaler
  TIMSK1 |= (1 << OCIE1A);  // enable timer compare interrupt

  // Activez registrii necesari pentru intreruperea generata la apasarea butonului.
  EIMSK |= (1 << INT0);
  EICRA |= (1 << ISC00);
  
  sei();
}

// Aici tratez rutina de intreruperi pentru timer.
ISR(TIMER1_COMPA_vect) {
  if ((contor >> 3) == 20) {
    rulare_joc = false;
  } else if (rulare_joc == true) {
    contor++;
  }
}

// Aici tratez rutina de intreruperi pentru buton.
ISR(INT0_vect) {
   int buttonState = digitalRead(PIN_BUTON);
   
   if (buttonState == HIGH) {
      buton_activat = 0;
      mesaj_inceput_joc = false;
   } else {
      buton_activat = 1;
   }
}

void loop(){
  static byte pozitie_jucator = POZITIE_JUCATOR_JOS_1;
  static byte obstacol_urmator = CELULA_GOALA;
  static byte timp_de_viata_obstacol = 1;
  
  if (rulare_joc == false) {
    initializare_harta();
    if ((contor >> 3) == 20) {
      lcd.clear();
      lcd.print("You won!");
      delay(2000);
      contor = 0;
    }

    lcd.clear();
    lcd.print("Press Start");
    
    if (buton_activat == 1) {
      buton_activat = 0;
      pozitie_jucator = POZITIE_JUCATOR_JOS_1;
      rulare_joc = true;
      contor = 0;
    }

    if (thisNote == notes * 2) {
      thisNote = 0;  
    }
    
    // calculates the duration of each note
    divider = melody[thisNote + 1];
    if (divider > 0) {
      // regular note, just proceed
      noteDuration = (wholenote) / divider;
    } else if (divider < 0) {
      // dotted notes are represented with negative durations!!
      noteDuration = (wholenote) / abs(divider);
    }

    // we only play the note for 90% of the duration, leaving 10% as a pause
    tone(buzzer, melody[thisNote], noteDuration * 0.9);

    // Wait for the specief duration before playing the next note.
    delay(noteDuration * 0.9);

    // stop the waveform generation before the next note.
    noTone(buzzer);

    thisNote += 2;
    
    return;
  }
  
  
  // Adaug un obstacol nou din partea dreapta.
  timp_de_viata_obstacol -= 1;
  if (timp_de_viata_obstacol == 0) {
    if (obstacol_urmator == 0) {
      if (random(2) != 0) {
        obstacol_urmator = BLOC_JOS;  
      } else {
        obstacol_urmator = BLOC_SUS;
      }
      
      timp_de_viata_obstacol = 2 + random(7);
    } else {
      obstacol_urmator = 0;
      timp_de_viata_obstacol = 7 + random(7);
    }
  }

  // Mut obstacolele la stanga
  if (obstacol_urmator == BLOC_JOS) {
    animatieHarta(harta_jos, BLOC_COMPLET);   
  } else {
    animatieHarta(harta_jos, CELULA_GOALA);   
  }

  if (obstacol_urmator == BLOC_SUS) {
    animatieHarta(harta_sus, BLOC_COMPLET);   
  } else {
    animatieHarta(harta_sus, CELULA_GOALA);
  }
    
  if (buton_activat == 1) {
    if (pozitie_jucator <= POZITIE_JUCATOR_JOS_2) {
      pozitie_jucator = POZITIE_JUCATOR_ALERGARE_1;
    }

    buton_activat = 0;
  }  
  if (reprezentare_jucator(pozitie_jucator, harta_sus, harta_jos, contor >> 3) == 1) {
    // Daca are loc o coliziune, jucatorul se opreste.
    rulare_joc = false;

    lcd.clear();
    lcd.print("You lose!");
    
    delay(2000);
  } else {
    if (pozitie_jucator == POZITIE_JUCATOR_JOS_2 || pozitie_jucator == POZITIE_JUCATOR_CAZATURA_3) {
      pozitie_jucator = POZITIE_JUCATOR_JOS_1;
    } else if ((pozitie_jucator >= POZITIE_JUCATOR_SARITURA_1 && pozitie_jucator <= POZITIE_JUCATOR_SARITURA_3) 
                && harta_jos[POZITIE_ORIZONTALA_JUCATOR] != CELULA_GOALA) {
      pozitie_jucator = POZITIE_JUCATOR_SUS_1;
    } else if (pozitie_jucator >= POZITIE_JUCATOR_SUS_1 && harta_jos[POZITIE_ORIZONTALA_JUCATOR] == CELULA_GOALA) {
      pozitie_jucator = POZITIE_JUCATOR_SARITURA_3;
    } else if (pozitie_jucator == POZITIE_JUCATOR_SUS_2) {
      pozitie_jucator = POZITIE_JUCATOR_SUS_1;
    } else {
      pozitie_jucator++;
    }

    if (harta_jos[POZITIE_ORIZONTALA_JUCATOR + 2] == CELULA_GOALA) {
      digitalWrite(PIN_AUTOPLAY, HIGH);   
    } else {
      digitalWrite(PIN_AUTOPLAY, LOW);    
    }
  }
  
  delay(100);
}
